/***************************************************************************
    AutoTextArea - TextArea with word wrap

    author:  Ulrich Krebs
    email :  ukrebs@freenet.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

class AutoTextArea extends TextArea implements ComponentListener
{
	boolean internalUpdate=false;
	String orgText;
	
	public AutoTextArea()
	{
		super();
		setEditable(false);
		setBackground(Color.white);
		addComponentListener(this);
	}
	
	public AutoTextArea(int rows, int columns)
	{
		super(rows,columns);
		setEditable(false);
		setBackground(Color.white);
		addComponentListener(this);
	}

	public AutoTextArea(String text)
	{
		super(text);
		setEditable(false);
		setBackground(Color.white);
		orgText=text;
		addComponentListener(this);
	}

	public AutoTextArea(String text, int rows, int columns)
	{
		super(text,rows,columns);
		setEditable(false);
		setBackground(Color.white);
		orgText=text;
		addComponentListener(this);
	}

	public AutoTextArea(String text, int rows, int columns, int scrollbars)
	{
		super(text,rows,columns,scrollbars);
		setEditable(false);
		setBackground(Color.white);
		orgText=text;
		addComponentListener(this);
	}
	
	public void setText(String text)
	{
		if(!internalUpdate)
		{
			orgText=text;
			text=reformat(text);
		}
		super.setText(text);
	}
	
	public String getText()
	{
		return orgText;
	}

	private String reformat(String text)
	{
		StringBuffer formattedText=new StringBuffer();
		StringBuffer buf;
		
		if(text==null) return null;
		
		int width=getSize().width-40;
		
		FontMetrics fm=getFontMetrics(getFont());

		BufferedReader br=new BufferedReader(new StringReader(text));
		String line;
		String separator="\n";

		try
		{
			while((line=br.readLine())!=null)
			{
				StringTokenizer wt=new StringTokenizer(line," ");
				buf=new StringBuffer();
				while(wt.hasMoreTokens())
				{
					String word=wt.nextToken();
					int textWidth=fm.stringWidth(buf+" "+word);
					if(textWidth>width)
					{
						formattedText.append(buf);
						formattedText.append(separator);
						buf=new StringBuffer(word);
					}
					else
					{
						if(buf.length()>0) buf.append(" ");
						buf.append(word);
					}
				}
				formattedText.append(buf);
				formattedText.append(separator);
			}
			br.close();
		}
		catch(IOException e)
		{
		}
		return formattedText.toString();
	}
		
	public void componentHidden(ComponentEvent e)
	{
	}
	
	public void componentMoved(ComponentEvent e)
	{
	}
	
	public void componentResized(ComponentEvent e)
	{
		internalUpdate=true;
		setText(reformat(orgText));
		internalUpdate=false;
	}
	
	public void componentShown(ComponentEvent e)
	{
	}
}